#include <stdio.h>
#include <xchat-plugin.h>
#include <glib.h>

/* plugin handle */
static xchat_plugin *ph;  

/* is backbiter enabled? */
gboolean is_enabled=FALSE;

/* sender (user) */
char *sender = NULL;

/* recipient (user or channel) */
char *recipient = NULL;

// -------------------------------------------------------------------------
// Send the message to the recipient/channel
// -------------------------------------------------------------------------
static void talk_now(char *message, char *its_from_me)
{
  if (its_from_me == NULL)
    xchat_commandf(ph, "msg %s <%s> %s", recipient, sender, message);
  else
    xchat_commandf(ph, "msg %s <%s> %s", recipient, its_from_me, message);
}

// -------------------------------------------------------------------------
// send my private message from query 
// -------------------------------------------------------------------------
static int my_private_msg (char *word[], void *userdata) 
{
  char *its_from_me;
  if (is_enabled)
  {
    if (strcmp(xchat_get_info(ph,"channel"), sender) == 0)
    {
      its_from_me = g_strdup(xchat_get_info(ph, "nick"));
      talk_now(word[2], its_from_me);
    }
  }
  return XCHAT_EAT_NONE;
}

// -------------------------------------------------------------------------
// send the private message from query from the other
// -------------------------------------------------------------------------
static int query_private_msg (char *word[], void *userdata) 
{
  if (is_enabled)
  {
    if (xchat_nickcmp(ph, word[1], sender) == 0)
      talk_now(word[2], NULL);
  }
  return XCHAT_EAT_NONE;
}

// -------------------------------------------------------------------------
// start backbiter
// -------------------------------------------------------------------------
static int backbiter_start(char *from, char *to)
{
  if (is_enabled == 0 && sender == NULL && recipient == NULL)
  {
    if (strcmp(from,"") != 0 && strcmp(to,"") != 0)
    {
      sender = g_strdup(from);
      recipient = g_strdup(to);
      is_enabled = 1;

      xchat_printf(ph, 
		   "Backbiter started successfully: \n"
		   "  ==>  Sending Messages from %s to %s.", 
		   sender, recipient);
    }
    else
      xchat_print(ph, "Usage: backbiter start #sender #recipient");
  }
  else
    xchat_print(ph, "Backbiter already started");    
    
  return 1;
}

// -------------------------------------------------------------------------
// stop backbiter
// -------------------------------------------------------------------------
static int backbiter_stop()
{
  if (sender != NULL && recipient != NULL)
  {
    if (is_enabled==FALSE)
      xchat_print (ph, "Halting Backbiter in paused modus...");
    
    xchat_printf(ph, 
		 "Backbiter halted successfully: \n"
		 "  ==>  Stop sending Messages from %s to %s.", 
		 sender, recipient);
    
    sender=NULL;
    recipient=NULL;
    is_enabled=FALSE;
  }
  else
    xchat_print(ph,"backbiter is halted - start it first");
  
  return XCHAT_EAT_ALL;
}

// -------------------------------------------------------------------------
// backbiter pause/resume
// -------------------------------------------------------------------------
static int backbiter_pause(gboolean pause) // TRUE if you want set pause
{
  if (pause)
  {
    if (is_enabled)
    {
      is_enabled=FALSE;
      xchat_printf(ph, 
		   "Backbiter paused successfully: \n"
		   "  ==>  Pause sending Messages from %s to %s.",   
		   sender, recipient);  
    }
    else
    {
      if (sender == NULL && recipient == NULL)
	xchat_print(ph,"backbiter is halted - start it first");
      else
	xchat_print(ph,"backbiter already paused");
    }
  }
  else
  {
    if (is_enabled == FALSE)
    {
      if (sender != NULL && recipient != NULL)
      {
	is_enabled=TRUE;
	xchat_printf(ph, 
		     "Backbiter resumed successfully: \n"
		     "  ==>  Resume sending Messages from %s to %s.",   
		     sender, recipient);  
      }
      else
	xchat_print(ph,"backbiter is halted - start it first");
    }
    else
      xchat_print(ph,"backbiter is not paused - resume unnecessary :-)");
  }
  return 1;
}

// -------------------------------------------------------------------------
// backbiter main menu
// -------------------------------------------------------------------------
static int backbiter(char *word[], char *word_eol[], void *userdata)
{
  if (strcmp(word[2],"start") == 0)
  {
    backbiter_start(word[3], word[4]);
  }
  else if (strcmp(word[2],"stop") == 0)
  {
    backbiter_stop();
  }
  else if (strcmp(word[2],"pause") == 0)
  {
    backbiter_pause(TRUE); // pause it
  }
  else if (strcmp(word[2],"resume") == 0)
  {
    backbiter_pause(FALSE); // resume the pause
  }
  else
  {
    xchat_print(ph,
		"\nBackbiter Usage: \n"
		"  backbiter start #sender #recipient -Start backbiter\n"
		"  backbiter stop -Stop backbiter\n"
		"  backbiter pause - Pause this backbiter session\n"
		"  backbiter resume - Resume this backbiter session\n"
		"  backbiter help - Show this message\n\n"
		);
  }
  return 1;
}

// -------------------------------------------------------------------------
// Init xchat_plugin (Main function)
// -------------------------------------------------------------------------
int xchat_plugin_init(xchat_plugin *plugin_handle,
                      char **plugin_name,
                      char **plugin_desc,
                      char **plugin_version,
                      char *arg)
{
  /* we need to save this for use with any xchat_* functions */
   ph = plugin_handle;

   *plugin_name = "Backbiter";
   *plugin_desc = "Send all messages from one query to another channel/query";
   *plugin_version = "0.1";

   xchat_hook_command(ph, 
		      "backbiter",
		      XCHAT_PRI_NORM, 
		      backbiter,
		      "Usage: backbiter [start,stop,pause,resume,help]",
		      NULL);
   
   xchat_hook_print(ph, 
		    "Private Message to Dialog", 
		    XCHAT_PRI_NORM, query_private_msg, 
		    NULL);
   
   xchat_hook_print(ph, 
		    "Your Message", 
		    XCHAT_PRI_NORM, 
		    my_private_msg, 
		    NULL);

   return 1;       /* return 1 for success */
}
